@@ -1,6 +1,7 @@
 from botocore.awsrequest import prepare_request_dict
 from botocore.client import logger, PaginatorDocstring, ClientCreator, \
-    BaseClient, ClientEndpointBridge, S3ArnParamHandler, S3EndpointSetter
+    BaseClient, ClientEndpointBridge, S3ArnParamHandler, S3EndpointSetter, \
+    resolve_checksum_context, apply_request_checksum
 from botocore.discovery import block_endpoint_discovery_required_operations
 from botocore.exceptions import OperationNotPageableError
 from botocore.history import get_global_history_recorder
@@ -12,6 +13,7 @@ from .paginate import AioPaginator
 from .args import AioClientArgsCreator
 from .utils import AioS3RegionRedirector
 from .discovery import AioEndpointDiscoveryManager, AioEndpointDiscoveryHandler
+from .retries import adaptive
 from . import waiter
 
 history_recorder = get_global_history_recorder()
@@ -32,7 +34,8 @@ class AioClientCreator(ClientCreator):
             region_name, client_config)
         endpoint_bridge = ClientEndpointBridge(
             self._endpoint_resolver, scoped_config, client_config,
-            service_signing_name=service_model.metadata.get('signingName'))
+            service_signing_name=service_model.metadata.get('signingName'),
+            config_store=self._config_store)
         client_args = self._get_client_args(
             service_model, region_name, is_secure, endpoint_url,
             verify, credentials, scoped_config, client_config, endpoint_bridge)
@@ -41,6 +44,9 @@ class AioClientCreator(ClientCreator):
         self._register_s3_events(
             service_client, endpoint_bridge, endpoint_url, client_config,
             scoped_config)
+        self._register_s3_control_events(
+            service_client, endpoint_bridge, endpoint_url, client_config,
+            scoped_config)
         self._register_endpoint_discovery(
             service_client, endpoint_url, client_config
         )
@@ -86,18 +92,57 @@ class AioClientCreator(ClientCreator):
             events.register('before-parameter-build',
                             block_endpoint_discovery_required_operations)
 
+    def _register_retries(self, client):
+        # botocore retry handlers may block. We add our own implementation here.
+        # botocore provides three implementations:
+        #
+        # 1) standard
+        # This one doesn't block. A threading.Lock is used in quota.RetryQuota,
+        # but it's only used to protect concurrent modifications of internal
+        # state inside multithreaded programs. When running under a single
+        # asyncio thread, this lock will be acquired and released in the same
+        # coroutine, and the coroutine will never block waiting for the lock.
+        # Thus, we don't need to redefine this strategy.
+        #
+        # 2) adaptive
+        # This one blocks when the client is applying self rate limiting.
+        # We override the corresponding definition to replace it with async
+        # objects.
+        #
+        # 3) legacy
+        # This one probably doesn't block.
+        #
+        # The code for this method comes directly from botocore. We could
+        # override `_register_v2_adaptive_retries` only. The override for
+        # `_register_retries` is only included for clarity.
+        retry_mode = client.meta.config.retries['mode']
+        if retry_mode == 'standard':
+            self._register_v2_standard_retries(client)
+        elif retry_mode == 'adaptive':
+            self._register_v2_standard_retries(client)
+            self._register_v2_adaptive_retries(client)
+        elif retry_mode == 'legacy':
+            self._register_legacy_retries(client)
+
+    def _register_v2_adaptive_retries(self, client):
+        # See comment in `_register_retries`.
+        # Note that this `adaptive` module is an aiobotocore reimplementation.
+        adaptive.register_retry_handler(client)
+
     def _register_s3_events(self, client, endpoint_bridge, endpoint_url,
                             client_config, scoped_config):
         if client.meta.service_model.service_name != 's3':
             return
         AioS3RegionRedirector(endpoint_bridge, client).register()
         S3ArnParamHandler().register(client.meta.events)
+        use_fips_endpoint = client.meta.config.use_fips_endpoint
         S3EndpointSetter(
             endpoint_resolver=self._endpoint_resolver,
             region=client.meta.region_name,
             s3_config=client.meta.config.s3,
             endpoint_url=endpoint_url,
-            partition=client.meta.partition
+            partition=client.meta.partition,
+            use_fips_endpoint=use_fips_endpoint
         ).register(client.meta.events)
         self._set_s3_presign_signature_version(
             client.meta, client_config, scoped_config)
@@ -152,6 +197,7 @@ class AioBaseClient(BaseClient):
         }
         request_dict = await self._convert_to_request_dict(
             api_params, operation_model, context=request_context)
+        resolve_checksum_context(request_dict, operation_model, api_params)
 
         service_id = self._service_model.service_id.hyphenize()
         handler, event_response = await self.meta.events.emit_until_response(
@@ -164,6 +210,7 @@ class AioBaseClient(BaseClient):
         if event_response is not None:
             http, parsed_response = event_response
         else:
+            apply_request_checksum(request_dict)
             http, parsed_response = await self._make_request(
                 operation_model, request_dict, request_context)
 
